@@ -1,9 +1,9 @@
 use async_std::task::sleep;
-use clap::Clap;
+use clap::Parser;
 use std::time::Duration;
 use std::time::Instant;
 
-#[derive(Clap)]
+#[derive(Parser)]
 #[clap()]
 struct Args {
     #[clap(about = "Serial port to use")]
