@@ -1,62 +1,4 @@
-//! # Lynxmotion Smart Servo Driver
-//!
-//! [![Crates.io](https://img.shields.io/crates/v/lss_driver.svg)](https://crates.io/crates/lss_driver)
-//! [![Docs](https://docs.rs/lss_driver/badge.svg)](https://docs.rs/lss_driver)
-//! [![Rust](https://github.com/dmweis/lss_driver/workflows/Rust/badge.svg)](https://github.com/dmweis/lss_driver/actions)
-//! [![codecov](https://codecov.io/gh/dmweis/lss_driver/branch/main/graph/badge.svg)](https://codecov.io/gh/dmweis/lss_driver)
-//! [![License](https://img.shields.io/crates/l/lss_driver)](https://github.com/dmweis/lss_driver)
-//!
-//! This crate provides an asynchronous serial driver the the Lynxmotion smart servos.  
-//!
-//! You can read more about the servos on the official [robotshop wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/)  
-//!
-//! ## Driver
-//!
-//! The Smart servos are controlled over a serial UART protocol.  
-//!
-//! You can read about the protocol [here](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/)
-//! It's highly recommended to at least skim the protocol page before trying to use this driver.  
-//!
-//! This driver currently doesn't implement all features from the protocol.  
-//! Some missing features are modifiers and setup commands.  
-//! If there are any missing commands or features you'd like added feel free to raise a PR or an issue.  
-//!
-//! This driver uses `async/await`. As a result you will need to use an async runtime.
-//! The driver is based on [tokio-serial](https://github.com/berkowski/tokio-serial) so tokio would be a good choice but any should work.
-//!
-//! ## Usage
-//!
-//! This crate comes with multiple [examples](https://github.com/dmweis/lss_driver/tree/main/examples).  
-//! These are a good start if you want to learn how to use it.  
-//!
-//! ```no_run
-//!
-//! #[tokio::main]
-//! async fn main() {
-//!     // Create a driver on port `COM14` or `/dev/ttyUSB0`...
-//!     let mut driver = lss_driver::LSSDriver::new("COM14").unwrap();
-//!     // In case there is only one servo connected
-//!     // we can query it's ID using the broadcast ID
-//!     let id = driver.query_id(lss_driver::BROADCAST_ID).await.unwrap();
-//!     // move motor with ID 5 to 90.0 degrees
-//!     driver.move_to_position(5, 90.0).await.unwrap();
-//!     // Set color of servo with ID 5 to Magenta
-//!     driver.set_color(5, lss_driver::LedColor::Magenta).await.unwrap();
-//! }
-//!
-//! ```
-//!
-//! ## Building
-//!
-//! This package shouldn't depend on any native libraries.  
-//! Rust [serialport](https://gitlab.com/susurrus/serialport-rs) depends on `pkg-config` and `libudev-dev` on GNU Linux but they should be disabled for this crate.  
-//! If you do run into issues with them failing it may be worth looking into their dependencies and raising an issue here.  
-//!
-//! ## Disclaimer
-//!
-//! _This software is not officially endorsed by Lynxmotion or Robotshop!_
-//!
-//! All product names, logos, and brands are property of their respective owners. All company, product and service names used in this website are for identification purposes only. Use of these names, logos, and brands does not imply endorsement.
+#![doc = include_str!("../README.md")]
 
 mod message_types;
 mod serial_driver;
@@ -821,6 +763,243 @@ impl LSSDriver {
             .await?;
         Ok(())
     }
+
+    /// Query origin offset in degrees
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HOriginOffset28O29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of servo you want to query
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     let origin_offset = driver.query_origin_offset(5).await;
+    /// }
+    /// ```
+    pub async fn query_origin_offset(&mut self, id: u8) -> DriverResult<f32> {
+        // response messages looks like *5QO-13
+        // Response is in tenths of degrees
+        self.driver.send(LssCommand::simple(id, "QO")).await?;
+        let response = self.driver.receive().await?;
+        let (_, value) = response.separate("QO")?;
+        Ok(value as f32 / 10.0)
+    }
+
+    /// Query the angular range in degrees
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HAngularRange28AR29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to query
+    ///
+    ///  /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     let angular_range = driver.query_angular_range(5).await.unwrap();
+    /// }
+    /// ```
+    pub async fn query_angular_range(&mut self, id: u8) -> DriverResult<f32> {
+        // Response looks like *5QAR1800 where 1800 is range in 1/10 degrees
+        // Contrary what the wiki says, which is *5AR1800 as example, the servo I used (HT1) returns
+        // *5QAR1800
+        self.driver.send(LssCommand::simple(id, "QAR")).await?;
+
+        let response = self.driver.receive().await?;
+        let (_, value) = response.separate("QAR")?;
+
+        Ok(value as f32 / 10.0)
+    }
+
+    /// Set the angular range in degrees
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HAngularRange28AR29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to control
+    /// * `range` - Angular range in degrees
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     driver.set_angular_range(5, 180.0).await;
+    /// }
+    pub async fn set_angular_range(&mut self, id: u8, range: f32) -> DriverResult<()> {
+        self.driver
+            .send(LssCommand::with_param(id, "CAR", (range * 10.) as i32))
+            .await?;
+
+        Ok(())
+    }
+
+    /// Queries the position in µs.
+    ///
+    /// When the position is inside the angular range it will return a value between 500 and 2500,
+    /// when the position is outside the angular range it will return either -500 or -2500.
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HPositioninPWM28P29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to control
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     let pwm_position = driver.query_pwm_position(5).await.unwrap();
+    /// }
+    /// ```
+    pub async fn query_pwm_position(&mut self, id: u8) -> DriverResult<i32> {
+        // Response looks like *5QP2334 where 2335 is in µs
+        self.driver.send(LssCommand::simple(id, "QP")).await?;
+
+        let response = self.driver.receive().await?;
+        let (_, value) = response.separate("QP")?;
+
+        Ok(value)
+    }
+
+    /// Set origin offset in degrees
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HOriginOffset28O29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of servo you want to control
+    /// * `origin_offset` - Offset from factory 0 in degrees
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     driver.set_origin_offset(5, -1.3).await;
+    /// }
+    /// ```
+    pub async fn set_origin_offset(&mut self, id: u8, origin_offset: f32) -> DriverResult<()> {
+        self.driver
+            .send(LssCommand::with_param(
+                id,
+                "CO",
+                (origin_offset * 10.) as i32,
+            ))
+            .await?;
+        Ok(())
+    }
+
+    /// Move to PWM position in µs.
+    ///
+    /// You can use [set_angular_range](LSSDriver::set_angular_range) to range.
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HPositioninPWM28P29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to control
+    /// * `position` - Position in µs in the range [500,2500]
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::LSSDriver;
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     driver.move_to_pwm_position(5, 2334).await;
+    /// }
+    /// ```
+    pub async fn move_to_pwm_position(&mut self, id: u8, position: i32) -> DriverResult<()> {
+        self.driver
+            .send(LssCommand::with_param(id, "P", position))
+            .await?;
+
+        Ok(())
+    }
+
+    /// Move to PWM position in µs with modifier.
+    ///
+    /// You can use [set_angular_range](LSSDriver::set_angular_range) to range.
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HPositioninPWM28P29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to control
+    /// * `position` - Position in µs in the range [500,2500]
+    /// * `modifier` - Modifier applied to this motion. Look at the type for more info.
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::{LSSDriver, CommandModifier};
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     driver.move_to_pwm_position_with_modifier(5, 2334, CommandModifier::Speed(750)).await;
+    /// }
+    /// ```
+    pub async fn move_to_pwm_position_with_modifier(
+        &mut self,
+        id: u8,
+        position: i32,
+        modifier: CommandModifier,
+    ) -> DriverResult<()> {
+        self.driver
+            .send(LssCommand::with_param_modifier(id, "P", position, modifier))
+            .await?;
+
+        Ok(())
+    }
+
+    /// Move to PWM position in µs with modifiers.
+    ///
+    /// You can use [set_angular_range](LSSDriver::set_angular_range) to range.
+    ///
+    /// Read more on the [wiki](https://www.robotshop.com/info/wiki/lynxmotion/view/lynxmotion-smart-servo/lss-communication-protocol/#HPositioninPWM28P29)
+    ///
+    /// # Arguments
+    ///
+    /// * `id` - ID of the servo you want to control
+    /// * `position` - Position in µs in the range [500,2500]
+    /// * `modifiers` - Array of modifiers applied to this motion. Look at the type for more info.
+    ///
+    /// # Example
+    /// ```no_run
+    /// use lss_driver::{LSSDriver, CommandModifier};
+    ///
+    /// async fn async_main() {
+    ///     let mut driver = LSSDriver::with_baud_rate("COM1", 115200).unwrap();
+    ///     driver.move_to_pwm_position_with_modifiers(5, 2334, &[CommandModifier::Speed(750), CommandModifier::Timed(2500)]).await;
+    /// }
+    /// ```
+    pub async fn move_to_pwm_position_with_modifiers(
+        &mut self,
+        id: u8,
+        position: i32,
+        modifiers: &[CommandModifier],
+    ) -> DriverResult<()> {
+        self.driver
+            .send(LssCommand::with_param_modifiers(
+                id, "P", position, modifiers,
+            ))
+            .await?;
+
+        Ok(())
+    }
 }
 
 #[cfg(test)]
@@ -1285,4 +1464,72 @@ mod tests {
         "#254RESET\r",
         |mut driver: LSSDriver| async move { driver.reset(BROADCAST_ID).await.unwrap() }
     );
+
+    test_query_float!(
+        test_query_origin_offset,
+        "#5QO\r",
+        "*5QO-13\r",
+        |mut driver: LSSDriver| async move { driver.query_origin_offset(5).await.unwrap() },
+        -1.3
+    );
+
+    test_command!(
+        test_set_origin_offset,
+        "#5CO-24\r",
+        |mut driver: LSSDriver| async move { driver.set_origin_offset(5, -2.4).await.unwrap() }
+    );
+
+    test_query_float!(
+        test_query_angular_range,
+        "#5QAR\r",
+        "*5QAR1800\r",
+        |mut driver: LSSDriver| async move { driver.query_angular_range(5).await.unwrap() },
+        180.0
+    );
+
+    test_command!(
+        test_set_angular_range,
+        "#5CAR1800\r",
+        |mut driver: LSSDriver| async move { driver.set_angular_range(5, 180.0).await.unwrap() }
+    );
+
+    test_query!(
+        test_query_pwm_position,
+        "#5QP\r",
+        "*5QP2334\r",
+        |mut driver: LSSDriver| async move { driver.query_pwm_position(5).await.unwrap() },
+        2334
+    );
+
+    test_command!(
+        test_move_to_pwm_position,
+        "#5P2334\r",
+        |mut driver: LSSDriver| async move { driver.move_to_pwm_position(5, 2334).await.unwrap() }
+    );
+
+    test_command!(
+        test_move_to_pwm_position_with_modifier,
+        "#5P2334S750\r",
+        |mut driver: LSSDriver| async move {
+            driver
+                .move_to_pwm_position_with_modifier(5, 2334, CommandModifier::Speed(750))
+                .await
+                .unwrap()
+        }
+    );
+
+    test_command!(
+        test_move_to_pwm_position_with_modifiers,
+        "#5P2334S750T2500\r",
+        |mut driver: LSSDriver| async move {
+            driver
+                .move_to_pwm_position_with_modifiers(
+                    5,
+                    2334,
+                    &[CommandModifier::Speed(750), CommandModifier::Timed(2500)],
+                )
+                .await
+                .unwrap()
+        }
+    );
 }
